@@ -12,14 +12,11 @@ from typing import Any, Optional, Union
 from . import LATEX_CONFIG_DIC, get_logger
 from .tools.template_strings import (
     FigureTemplate,
+    InputTemplate,
     SiUnitxTemplate,
     TableTemplate,
 )
 
-LOGGER = get_logger(__name__)
-"""logging.Logger: The module level logger.
-"""
-
 
 def format_table(number: Union[int, float], unit: Optional[str] = None) -> str:
     r"""Format numbers in tables.
@@ -62,6 +59,7 @@ class Chapter(object):
         chapter_dir: pathlib.Path,
         location: str,
         typ: str,
+        stream: bool = False,
     ) -> None:
         r"""Init the class.
 
@@ -87,21 +85,26 @@ class Chapter(object):
             * "tabs".
             * "code".
 
+        stream : bool
+            Determine whether to log to console or to a file,
+            by default False.
+
         """
         # save the chapter directory
         self._chapter_dir: pathlib.Path = chapter_dir
         # save the location where it should be saved
-        self._location: list[str] = (
-            location.lower().replace(" ", "").split("\n")
+        self._location: list[str] = location.lower().replace(" ", "").split(
+            "\n"
         )
-        # define a template string with the LaTeX input command
-        self._inputstr = string.Template("\n\\input{$path}\n")
         # get the filename
-        self._filename = filename
+        self._filename: str = filename
         # get the file format
         self._fmt: str = filename.split(".")[-1].lower()
         # get the corresponding folder
         self._folder: str = typ.lower()
+        # define the logger
+        self._stream = stream
+        self._logger = get_logger(type(self).__name__, stream=stream)
 
     def __str__(self) -> str:
         """String representation of ``self``.
@@ -144,27 +147,30 @@ class Chapter(object):
 
     def _fill_template(
         self,
-        fname: pathlib.Path,
+        path: pathlib.Path,
         template_str: string.Template,
-        template_desc: dict[str, Union[float, str, bool]],
+        template_desc: dict[str, Union[float, str, bool, pathlib.Path]],
     ) -> None:
         """Fill the LaTeX template.
 
         Parameters
         ----------
-        fname : pathlib.Path
+        path : pathlib.Path
             The file which should be created based on ``template_str``.
         template_str : string.Template
             The template string.
-        template_desc : dict[str, Union[float, str, bool]]
+        template_desc : dict[str, Union[float, str, bool, pathlib.Path]]
             The fields to write to the template.
 
         """
         template = template_str.substitute(template_desc)
-        fname.write_text(template, encoding=LATEX_CONFIG_DIC["encoding"])
+        path.write_text(template, encoding=LATEX_CONFIG_DIC["encoding"])
 
     def save_fig(
-        self, fig: Any, fig_desc: dict[str, Union[float, str, bool]],
+        self,
+        fig: Any,
+        fig_desc: dict[str, Union[float, str, bool, pathlib.Path]],
+        **kwargs,
     ) -> None:
         r"""Save a result to a figure.
 
@@ -172,7 +178,7 @@ class Chapter(object):
         ----------
         fig : Union[matplotlib.figure.Figure, plotly.graph_objs._figure.Figure]
             The figure which should be saved.
-        fig_desc : dict[str, Union[float, str, bool]]
+        fig_desc : dict[str, Union[float, str, bool, pathlib.Path]]
             The discription of the figure, see also [1].
             The most important args are:
 
@@ -186,16 +192,33 @@ class Chapter(object):
             * "position": The LaTeX positional argument for tables,
                 to be placed after ``\begin{}`` in the output.
 
+        Other Parameters
+        ----------------
+        Parameters passed to either:
+
+            1. Matplotlib ``savefig`` method, see [2].
+            2. Plotly ``write_image`` method, see [3].
+
         Raises
         ------
+        ImportError
+            If the format is "eps" and the [poppler](https://github.com/cbrunet/python-poppler)
+            library is not installed if plotly is used.
         TypeError
             If the ``fig`` argument is neither of type
             ``matplotlib.figure.Figure`` nor of type
             ``plotly.graph_objs._figure.Figure``.
 
+        Notes
+        -----
+        - If [Kaleido](https://github.com/plotly/Kaleido) is installed,
+            then the ``**kwargs`` argument is currently not used.
+
         References
         ----------
         [1] https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_latex.html
+        [2] https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.savefig.html
+        [3] https://plotly.github.io/plotly.py-docs/generated/plotly.graph_objects.Figure.html#write_image
 
         """
         # construct the corresponding path
@@ -208,17 +231,23 @@ class Chapter(object):
         child_filename_tex = child_folder / self._filename.replace(
             self._fmt, "tex"
         )
+        fig_desc["path"] = child_filename
         t = type(fig)
         # check if t is of type matplotlib figure
         if t.__module__ == (mm := "matplotlib.figure"):
             fig.savefig(
-                child_filename,
-                dpi=600,
-                orientation="portrait",
-                format=self._fmt,
+                child_filename, format=self._fmt, **kwargs,
             )
         # check if t is of type plotly figure
         elif t.__module__ == (mp := "plotly.graph_objs._figure"):
+            if self._fmt == "eps":
+                try:
+                    import poppler  # type: ignore
+                except ModuleNotFoundError as e:
+                    if not self._stream:
+                        msg: str = f"The poppler library needs to be installed when saving to eps format using plotly!"
+                        self._logger.critical(msg)
+                    raise ModuleNotFoundError from e
             try:
                 from kaleido.scopes.plotly import PlotlyScope  # type: ignore
 
@@ -228,16 +257,16 @@ class Chapter(object):
                 )
                 with child_filename.open(mode="wb") as file:
                     file.write(scope.transform(fig, format=self._fmt))
-            except ImportError:
-                LOGGER.debug(
+            except ModuleNotFoundError:
+                self._logger.warning(
                     f"Kaleido is not installed, falling back to plotly save."
                 )
-                fig.write_image(f"{child_filename}")
+                fig.write_image(f"{child_filename}", **kwargs)
         else:
-            raise TypeError(
-                f"fig is neither from matplotlib module {mm} nor from plotly module {mp}!"
-            )
-        fig_desc["fname"] = f"{child_filename}".replace("\\", "/")
+            msg: str = f"fig is neither from matplotlib module {mm} nor from plotly module {mp}!"
+            if not self._stream:
+                self._logger.critical(msg)
+            raise TypeError(msg)
         # fill the tex template
         if isinstance(fig_desc["caption"], tuple):
             short = True
@@ -336,11 +365,11 @@ class Chapter(object):
         n = data.shape[-1]
         column_type: Optional[Union[str, list[str]]] = latex_args.pop(
             "column_type", None
-        )
-        top_caption: bool = latex_args.pop("top_caption", False)
+        )  # type: ignore
+        top_caption: bool = latex_args.pop("top_caption", False)  # type: ignore
         data_str = self._rewrite_table(data_str, n, column_type, top_caption)
         latex_args["data"] = data_str
-        self._fill_template(child_filename, TableTemplate(), latex_args)
+        self._fill_template(child_filename, TableTemplate(), latex_args)  # type: ignore
         # get the tex filename for the parent
         parent_filename_tex = list(goal_dir.glob("*.tex"))[0]
         self.update(parent_filename_tex, child_filename)
@@ -399,11 +428,17 @@ class Chapter(object):
         [3] https://stackoverflow.com/questions/4703390/how-to-extract-a-floating-number-from-a-string
 
         """
-        col_re = re.compile("(?<=tabular}{).*?(?=})")
         if column_type:
-            assert (
-                n := len(column_type)
-            ) == num_data, f"The number of columns {n} does not match the number of columns in the data {num_data}!"
+            col_re = re.compile("(?<=tabular}{).*?(?=})")
+            try:
+                msg: str = "The number of columns {} does not match the number of columns in the data {}!"
+                assert (n := len(column_type)) == num_data, msg.format(
+                    n, num_data
+                )
+            except AssertionError as e:
+                if not self._stream:
+                    self._logger.critical(msg.format(n, num_data))
+                raise AssertionError from e
             if isinstance(column_type, list):
                 table_width: float = sum(
                     [
@@ -412,7 +447,7 @@ class Chapter(object):
                     ]
                 )
                 if table_width > (w := LATEX_CONFIG_DIC["scrbook_width"]):
-                    LOGGER.warning(
+                    self._logger.warning(
                         f"The table width {table_width}cm exceeds the limits by scrbook {w}cm!\n"
                     )
                 column_type_: str = "".join(column_type)
@@ -430,7 +465,7 @@ class Chapter(object):
             data = "\n".join(temp)
         return data
 
-    def update(self, parent: pathlib.Path, child: pathlib.Path,) -> None:
+    def update(self, parent: pathlib.Path, child: pathlib.Path) -> None:
         """Update the result.
 
         If the result is newly created but should still be located
@@ -445,21 +480,16 @@ class Chapter(object):
             The figure file which is included in the parent.
 
         """
-        # if only the figure should be updated but not the parent and the corresponding input
-        for i, part in enumerate(child.parts):
-            if part == "chapters":
-                # child is not unbound since the for loop breaks
-                child_ = "/".join(child.parts[i:])
-                break
-        string_ = self._inputstr.substitute(path=child_)  # type: ignore
+        template: string.Template = InputTemplate()
+        string_: str = template.substitute({"path": child})
         with parent.open(
             mode="r+", encoding=LATEX_CONFIG_DIC["encoding"]
         ) as file:
             temp: str = file.read()
             if string_ in temp:
-                LOGGER.debug(f"{string_} is already in {parent}!\n")
+                self._logger.debug(f"{string_} is already in {parent}!\n")
             else:
-                LOGGER.debug(
+                self._logger.debug(
                     f"{string_} is appended to the end of {parent}!\n"
                 )
                 file.write(string_)
